@@ -19,7 +19,7 @@ import pyff.index
 from pyff.logs import log
 from pyff.utils import schema, URLFetch, filter_lang, root, duration2timedelta, template
 import xmlsec
-from pyff.constants import NS, NF_URI
+from pyff.constants import NS, NF_URI, DIGESTS
 import traceback
 import threading
 from Queue import Queue
@@ -42,6 +42,10 @@ def _e(error_log):
     return "\n".join(filter(lambda x: ":WARNING:" not in x, ["%s" % e for e in error_log]))
 
 
+class MetadataException(Exception):
+    pass
+
+
 class MDRepository(DictMixin):
     """A class representing a set of SAML Metadata. Instances present as dict-like objects where
     the keys are URIs and values are EntitiesDescriptor elements containing sets of metadata.
@@ -188,9 +192,8 @@ The dict in the list contains three items:
 :param message: The ATOM content
 :param source: An optional source URL. It is added as a <link> element with @rel='saml-metadata-source'
         """
-        if e.tag != "{%s}EntityDescriptor" % NS['md'] and \
-                        e.tag != "{%s}EntitiesDescriptor" % NS['md']:
-            raise ValueError("I can only annotate EntityDescriptor or EntitiesDescriptor elements")
+        if e.tag != "{%s}EntityDescriptor" % NS['md'] and e.tag != "{%s}EntitiesDescriptor" % NS['md']:
+            raise MetadataException("I can only annotate EntityDescriptor or EntitiesDescriptor elements")
         subject = e.get('Name', e.get('entityID', None))
         atom = ElementMaker(nsmap={'atom': 'http://www.w3.org/2005/Atom'}, namespace='http://www.w3.org/2005/Atom')
         args = [atom.published("%s" % datetime.now().isoformat()),
@@ -232,10 +235,10 @@ The dict in the list contains three items:
 :param e: The EntityDescriptor element
 :param d: A dict of attribute-value pairs that should be added as entity attributes
 :param nf: The nameFormat (by default "urn:oasis:names:tc:SAML:2.0:attrname-format:uri") to use.
-:raise: ValueError unless e is an EntityDescriptor element
+:raise: MetadataException unless e is an EntityDescriptor element
         """
         if e.tag != "{%s}EntityDescriptor" % NS['md']:
-            raise ValueError("I can only add EntityAttribute(s) to EntityDescriptor elements")
+            raise MetadataException("I can only add EntityAttribute(s) to EntityDescriptor elements")
 
         #log.debug("set %s" % d)
         for attr, value in d.iteritems():
@@ -247,7 +250,7 @@ The dict in the list contains three items:
             a.append(velt)
             #log.debug(etree.tostring(a))
 
-    def fetch_metadata(self, resources, qsize=5, timeout=60, stats={}, xrd=None):
+    def fetch_metadata(self, resources, qsize=5, timeout=60, stats=None, xrd=None):
         """Fetch a series of metadata URLs and optionally verify signatures.
 
 :param resources: A list of triples (url,cert-or-fingerprint,id)
@@ -267,6 +270,8 @@ elements with a X509Certificate and where the <Rel> element contains the string
 'urn:oasis:names:tc:SAML:2.0:metadata', the corresponding <URL> element is download
 and verified.
         """
+        if stats is None:
+            stats = {}
 
         def producer(q, resources, cache=self.metadata_cache_enabled):
             print resources
@@ -276,7 +281,11 @@ and verified.
                 thread.start()
                 q.put(thread, True)
 
-        def consumer(q, njobs, stats, next_jobs=[], resolved=set()):
+        def consumer(q, njobs, stats, next_jobs=None, resolved=None):
+            if next_jobs is None:
+                next_jobs = []
+            if resolved is None:
+                resolved = set()
             nfinished = 0
 
             while nfinished < njobs:
@@ -287,7 +296,7 @@ and verified.
                     thread.join(timeout)
 
                     if thread.isAlive():
-                        raise ValueError("Thread timeout occured")
+                        raise MetadataException("Thread timeout occured")
 
                     info = {
                         'Time Spent': thread.time()
@@ -299,7 +308,7 @@ and verified.
                         if thread.result is not None:
                             info['Bytes'] = len(thread.result)
                         else:
-                            raise ValueError("Empty response")
+                            raise MetadataException("Empty response")
                         info['Cached'] = thread.cached
                         info['Date'] = str(thread.date)
                         info['Last-Modified'] = str(thread.last_modified)
@@ -312,7 +321,7 @@ and verified.
 
                     t = self.parse_metadata(StringIO(xml), key=thread.verify, base_url=thread.url)
                     if t is None:
-                        raise ValueError("No valid metadata found at %s" % thread.url)
+                        raise MetadataException("No valid metadata found at %s" % thread.url)
 
                     relt = root(t)
                     if relt.tag in ('{%s}XRD' % NS['xrd'], '{%s}XRDS' % NS['xrd']):
@@ -337,7 +346,7 @@ and verified.
 
                         if thread.cached:
                             if thread.last_modified + offset < datetime.now() - duration2timedelta(self.min_cache_ttl):
-                                raise ValueError("Cached metadata expired")
+                                raise MetadataException("Cached metadata expired")
                             else:
                                 log.debug("Found cached metadata (last-modified: %s)" % thread.last_modified)
                                 ne = self.import_metadata(t, url=thread.id)
@@ -353,10 +362,10 @@ and verified.
                             cert = certs.values()[0].strip()
                         resolved.add((thread.url, cert))
                     else:
-                        raise ValueError("Unknown metadata type (%s)" % relt.tag)
+                        raise MetadataException("Unknown metadata type (%s)" % relt.tag)
                 except Exception, ex:
-                    traceback.print_exc()
-                    log.error("Error fetching %s." % ex)
+                    traceback.print_exc(ex)
+                    log.error("Error fetching metadata: %s" % ex)
                     if info is not None:
                         info['Exception'] = ex
                     if thread.tries < self.retry_limit:
@@ -368,10 +377,11 @@ and verified.
                     if info is not None:
                         stats[thread.url] = info
 
-        resources = [(url, verify, id, 0) for url, verify, id in resources]
+        resources = [(url, verify, rid, 0) for url, verify, rid in resources]
         resolved = set()
         cache = True
         while len(resources) > 0:
+            log.debug("fetching %d resources (%s)" % (len(resources), repr(resources)))
             next_jobs = []
             q = Queue(qsize)
             prod_thread = threading.Thread(target=producer, args=(q, resources, cache))
@@ -380,6 +390,7 @@ and verified.
             cons_thread.start()
             prod_thread.join()
             cons_thread.join()
+            log.debug("after fetch: %d jobs to retry" % len(next_jobs))
             if len(next_jobs) > 0:
                 resources = next_jobs
                 cache = False
@@ -404,7 +415,7 @@ and verified.
             schema().assertValid(t)
         except DocumentInvalid, ex:
             log.debug(_e(ex.error_log))
-            raise ValueError("XML schema validation failed")
+            raise MetadataException("XML schema validation failed")
         except Exception, ex:
             log.debug(_e(schema().error_log))
             log.error(ex)
@@ -423,6 +434,12 @@ and verified.
 
         return t
 
+    def _index_entity(self, e):
+        #log.debug("adding %s to index" % e.get('entityID'))
+        if 'ID' in e.attrib:
+            del e.attrib['ID']
+        self.index.add(e)
+
     def import_metadata(self, t, url=None):
         """
 :param t: An EntitiesDescriptor element
@@ -437,7 +454,7 @@ is stored in the MDRepository object.
             if top is not None and len(top) == 1:
                 url = top[0].get("Name", None)
         if url is None:
-            raise ValueError("No collection name found")
+            raise MetadataException("No collection name found")
         self[url] = t
         # we always clean incoming ID
         # add to the index
@@ -445,15 +462,11 @@ is stored in the MDRepository object.
 
         if t is not None:
             if root(t).tag == "{%s}EntityDescriptor" % NS['md']:
-                if root(t).attrib.has_key('ID'):
-                    del root(t).attrib['ID']
-                self.index.add(root(t))
+                self._index_entity(root(t))
                 ne += 1
             else:
                 for e in t.findall(".//{%s}EntityDescriptor" % NS['md']):
-                    if e.attrib.has_key('ID'):
-                        del e.attrib['ID']
-                    self.index.add(e)
+                    self._index_entity(e)
                     ne += 1
 
         return ne
@@ -471,20 +484,20 @@ Returns the list of contained EntityDescriptor elements
         else:
             return t.findall(".//{%s}EntityDescriptor" % NS['md'])
 
-    def load_dir(self, dir, ext=".xml", url=None):
+    def load_dir(self, directory, ext=".xml", url=None):
         """
-:param dir: A directory to walk.
+:param directory: A directory to walk.
 :param ext: Include files with this extension (default .xml)
 
 Traverse a directory tree looking for metadata. Files ending in the specified extension are included. Directories
 starting with '.' are excluded.
         """
         if url is None:
-            url = dir
-        log.debug("walking %s" % dir)
-        if not self.md.has_key(dir):
+            url = directory
+        log.debug("walking %s" % directory)
+        if not directory in self.md:
             entities = []
-            for top, dirs, files in os.walk(dir):
+            for top, dirs, files in os.walk(directory):
                 for dn in dirs:
                     if dn.startswith("."):
                         dirs.remove(dn)
@@ -492,8 +505,11 @@ starting with '.' are excluded.
                     log.debug("found file %s" % nm)
                     if nm.endswith(ext):
                         fn = os.path.join(top, nm)
-                        t = self.parse_metadata(fn, fail_on_error=True)
-                        entities.extend(self.entities(t)) #local metadata is assumed to be ok
+                        try:
+                            t = self.parse_metadata(fn, fail_on_error=True)
+                            entities.extend(self.entities(t))  # local metadata is assumed to be ok
+                        except Exception, ex:
+                            log.error(ex)
             self.import_metadata(self.entity_set(entities, url))
         return self.md[url]
 
@@ -504,14 +520,14 @@ starting with '.' are excluded.
 Find a (set of) EntityDescriptor element(s) based on the specified 'member' expression.
         """
 
-        def _hash(hn, str):
+        def _hash(hn, strv):
             if hn == 'null':
-                return str
+                return strv
             if not hasattr(hashlib, hn):
-                raise ValueError("Unknown digest mechanism: '%s'" % hn)
+                raise MetadataException("Unknown digest mechanism: '%s'" % hn)
             hash_m = getattr(hashlib, hn)
             h = hash_m()
-            h.update(str)
+            h.update(strv)
             return h.hexdigest()
 
         if xp is None:
@@ -569,7 +585,7 @@ Find a (set of) EntityDescriptor element(s) based on the specified 'member' expr
                 return self.index.get(m.group(1), m.group(2).rstrip("/"))
 
             log.debug("basic lookup %s" % member)
-            for idx in ("null"):
+            for idx in DIGESTS:
                 e = self.index.get(idx, member)
                 if e:
                     log.debug("found %s in %s index" % (e, idx))
@@ -579,13 +595,13 @@ Find a (set of) EntityDescriptor element(s) based on the specified 'member' expr
             if e is not None:
                 return self._lookup(e, xp)
 
-            e = self.get("%s.xml" % member, None) #hackish but helps save people from their misstakes
+            e = self.get("%s.xml" % member, None)  # hackish but helps save people from their misstakes
             if e is not None:
-                if not "://" in member: # not an absolute URL
+                if not "://" in member:  # not an absolute URL
                     log.warn("Found %s.xml as an alias - AVOID extensions in 'select as' statements" % member)
                 return self._lookup(e, xp)
 
-            if "://" in member: # looks like a URL and wasn't an entity or collection - recurse away!
+            if "://" in member:  # looks like a URL and wasn't an entity or collection - recurse away!
                 log.debug("recursively fetching members from '%s'" % member)
                 # note that this supports remote lists which may be more rope than is healthy
                 return [self._lookup(line, xp) for line in urllib.urlopen(member).iterlines()]
@@ -596,7 +612,7 @@ Find a (set of) EntityDescriptor element(s) based on the specified 'member' expr
                 member = self.keys()
             return [self._lookup(m, xp) for m in member]
         else:
-            raise Exception, "What about %s ??" % member
+            raise MetadataException("What about %s ??" % member)
 
     def lookup(self, member, xp=None):
         """
@@ -646,7 +662,7 @@ Produce an EntityDescriptors set from a list of entities. Optional Name, cacheDu
             attrs['validUntil'] = validUntil
         t = etree.Element("{%s}EntitiesDescriptor" % NS['md'], **attrs)
         nent = 0
-        seen = {} # TODO make better de-duplication
+        seen = {}  # TODO make better de-duplication
         for member in entities:
             for ent in self.lookup(member):
                 entityID = ent.get('entityID', None)
@@ -665,7 +681,7 @@ Produce an EntityDescriptors set from a list of entities. Optional Name, cacheDu
                 schema().assertValid(t)
             except DocumentInvalid, ex:
                 log.debug(_e(ex.error_log))
-                raise ValueError("XML schema validation failed")
+                raise MetadataException("XML schema validation failed")
         return t
 
     def error_set(self, url, title, ex):
@@ -744,10 +760,10 @@ replace old_e in t.
                 module = getattr(__import__(pn, globals(), locals(), [modn], -1), modn)
             else:
                 module = __import__(mn, globals(), locals(), [], -1)
-            strategy = getattr(module, fn) # we might aswell let this fail early if the strategy is wrongly named
+            strategy = getattr(module, fn)  # we might aswell let this fail early if the strategy is wrongly named
 
         if strategy is None:
-            raise ValueError("No merge strategy - refusing to merge")
+            raise MetadataException("No merge strategy - refusing to merge")
 
         for e in nt.findall(".//{%s}EntityDescriptor" % NS['md']):
             entityID = e.get("entityID")
@@ -761,7 +777,8 @@ replace old_e in t.
                 #log.debug("removed old entity from index")
                 strategy(old_e, e)
                 new_e = t.find(".//{%s}EntityDescriptor[@entityID='%s']" % (NS['md'], entityID))
-                self.index.add(new_e) # we don't know which strategy was employed
+                if new_e:
+                    self.index.add(new_e)  # we don't know which strategy was employed
             except Exception, ex:
                 traceback.print_exc()
                 self.index.add(old_e)
