@@ -1,7 +1,9 @@
 """
 An implementation of draft-lajoie-md-query
 
-Usage: pyffd <options> {pipeline-files}+
+.. code-block:: bash
+
+    Usage: pyffd <options> {pipeline-files}+
 
     -C|--no-cache
             Turn off caching
@@ -52,14 +54,18 @@ from pyff.locks import ReadWriteLock
 from pyff.mdrepo import MDRepository
 from pyff.pipes import plumbing
 from pyff.utils import resource_string, template, xslt_transform, dumptree, duration2timedelta
-from pyff.logs import log, PyFFLogger, SysLogLibHandler
+from pyff.logs import log, SysLogLibHandler
 import logging
 from pyff.stats import stats
 import lxml.html as html
 from datetime import datetime
+from lxml import etree
+from pyff import __version__ as pyff_version
 
 __author__ = 'leifj'
 
+site_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "site")
+
 
 class MDUpdate(Monitor):
     def __init__(self, bus, frequency=600, server=None):
@@ -89,7 +95,7 @@ class MDUpdate(Monitor):
             else:
                 log.error("another instance is running - will try again later...")
         except Exception, ex:
-            traceback.print_exc()
+            traceback.print_exc(ex)
         finally:
             if locked:
                 self.lock.release()
@@ -116,6 +122,11 @@ class DirPlugin(SimplePlugin):
 
 
 class EncodingDispatcher(object):
+    """Cherrypy ass-u-me-s a lot about how requests are processed. In particular it is diffucult to send
+    something that contains '/' and ':' (like a URL) throught the standard dispatchers. This class provides
+    a workaround by base64-encoding the troubling stuff and sending the result through the normal displatch
+    pipeline. At the other end base64-encoded data is unpacked.
+    """
     def __init__(self, prefixes, enc, next_dispatcher=Dispatcher()):
         self.prefixes = prefixes
         self.enc = enc
@@ -134,12 +145,10 @@ class EncodingDispatcher(object):
         return self.next_dispatcher(vpath)
 
 
-site_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "site")
-
-from lxml import etree
-
-
 class MDStats(StatsPage):
+    """Renders the standard stats page with pyFF style decoration. We use the lxml html parser to locate the
+    body and replace it with a '<div>'. The result is passed as the content using the 'basic' template.
+    """
     @cherrypy.expose
     def index(self):
         h = "".join(super(MDStats, self).index())
@@ -147,16 +156,41 @@ class MDStats(StatsPage):
         tree = etree.parse(StringIO(h), parser)
         body = tree.getroot().find("body")
         body.tag = 'div'
-        str = etree.tostring(body, pretty_print=True, method="html")
-        return template("basic.html").render(content=str, http=cherrypy.request)
+        hstr = etree.tostring(body, pretty_print=True, method="html")
+        return template("basic.html").render(content=hstr, http=cherrypy.request)
 
 
 class WellKnown():
+    """Implementation of the .well-known URL namespace for pyFF. In particular this contains the webfinger
+    implementation which returns information about up- and downstream metadata.
+    """
     def __init__(self, server=None):
         self.server = server
 
     @cherrypy.expose
     def webfinger(self, resource=None, rel=None):
+        """An implementation the webfinger protocol (http://tools.ietf.org/html/draft-ietf-appsawg-webfinger-12)
+        in order to provide information about up and downstream metadata available at this pyFF instance.
+
+Example:
+
+.. code-block:: bash
+
+        # curl http://localhost:8080/.well-known/webfinger?resource=http://localhost:8080
+
+This should result in a JSON structure that looks something like this:
+
+.. code-block:: json
+
+        {"expires": "2013-04-13T17:40:42.188549",
+         "links": [
+            {"href": "http://reep.refeds.org:8080/role/sp.xml", "rel": "urn:oasis:names:tc:SAML:2.0:metadata"},
+            {"href": "http://reep.refeds.org:8080/role/sp.json", "rel": "disco-json"}],
+         "subject": "http://reep.refeds.org:8080"}
+
+Depending on which version of pyFF your're running and the configuration you may also see downstream metadata
+listed using the 'role' attribute to the link elements.
+        """
         if resource is None:
             raise cherrypy.HTTPError(400, "Bad Request - missing resource parameter")
 
@@ -169,7 +203,7 @@ class WellKnown():
 
         def _links(a):
             links.append(
-                dict(rel='urn:oasis:names:tc:SAML:2.0:metadata', href='%s/%s.xml' % (cherrypy.request.base, a)))
+                dict(rel='urn:oasis:names:tc:SAML:2.0:metadata', role="provider", href='%s/%s.xml' % (cherrypy.request.base, a)))
             links.append(dict(rel='disco-json', href='%s/%s.json' % (cherrypy.request.base, a)))
 
         for a in self.server.md.keys():
@@ -179,6 +213,7 @@ class WellKnown():
             elif 'http://' in a or 'https://' in a:
                 links.append(dict(rel='urn:oasis:names:tc:SAML:2.0:metadata',
                                   href=a,
+                                  role="consumer",
                                   properties=dict()))
 
         for a in self.server.aliases.keys():
@@ -190,6 +225,9 @@ class WellKnown():
 
 
 class MDRoot():
+    """The root application of pyFF. The root application assembles the MDStats and WellKnown classes with an
+    MDServer instance.
+    """
     def __init__(self, server):
         self.server = server
         self._well_known.server = server
@@ -200,25 +238,34 @@ class MDRoot():
     @cherrypy.expose
     @cherrypy.tools.expires(secs=3600, debug=True)
     def robots_txt(self):
+        """Returns a robots.txt that disables all robots.
+        """
         return """
 User-agent: *
 Disallow: /
 """
 
     @cherrypy.expose
-    @cherrypy.tools.expires(secs=3600, debug=True)
     def favicon_ico(self):
+        """Returns the pyff icon (the alchemic symbol for sublimation).
+        """
         cherrypy.response.headers['Content-Type'] = 'image/x-icon'
         return resource_string('favicon.ico', "site/static/icons")
 
     @cherrypy.expose
-    @cherrypy.tools.expires(secs=600, debug=True)
     def entities(self, path=None):
+        """Process an MDX request with Content-Type hard-coded to application/xml. Regardless of the suffix
+        you will get XML back from /entities/...
+        """
         return self.server.request(path=path, content_type="application/xml")
 
     @cherrypy.expose
-    @cherrypy.tools.expires(secs=600, debug=True)
     def metadata(self, path=None):
+        """The main request entry point. Any requests are subject to content negotiation based on Accept headers
+        and based on file name extension. Requesting /metadata/foo.xml gets you (signed) XML (assuming your pipeline
+        contains that mode), requesting /metadata/foo.json gets you json, and /metadata/foo.ds gets you a discovery
+        interface based on the IdPs found in 'foo'. Here 'foo' is any supported lookup expression.
+        """
         return self.server.request(path=path)
 
     @cherrypy.expose
@@ -227,6 +274,8 @@ Disallow: /
 
     @cherrypy.expose
     def about(self):
+        """The 'about' page. Contains links to statistics etc.
+        """
         import pkg_resources  # part of setuptools
 
         version = pkg_resources.require("pyFF")[0].version
@@ -260,14 +309,21 @@ Search the active set for matching entities.
 
     @cherrypy.expose
     def index(self):
+        """Alias for /metadata
+        """
         return self.server.request()
 
     @cherrypy.expose
     def static(self):
+        """Static resources in the main 'site' directory.
+        """
         return static.staticdir("/static", site_dir, debug=True)
 
     @cherrypy.expose
     def default(self, *args, **kwargs):
+        """The default request processor unpacks base64-encoded reuqests and passes them onto the MDServer.request
+        handler.
+        """
         log.debug("request default: %s" % ",".join(args))
         if len(args) > 0 and args[0] in self.server.aliases:
             kwargs['pfx'] = args[0]
@@ -287,8 +343,12 @@ Search the active set for matching entities.
 
 
 class MDServer():
+    """The MDServer class is the business logic of pyFF. This class is isolated from the request-decoding logic
+    of MDRoot and from the ancilliary classes like MDStats and WellKnown.
+    """
     def __init__(self, pipes=None, autoreload=False, frequency=600, aliases=ATTRS, cache_enabled=True):
-        if not pipes: pipes = []
+        if pipes is None:
+            pipes = []
         self.cache_enabled = cache_enabled
         self._md = None
         self.lock = ReadWriteLock()
@@ -328,6 +388,8 @@ class MDServer():
                 return False
 
     def request(self, **kwargs):
+        """The main request processor. This code implements all rendering of metadata.
+        """
         stats['MD Requests'] += 1
 
         pfx = kwargs.get('pfx', None)
@@ -377,7 +439,7 @@ class MDServer():
         else:
             q = path
 
-        logging.debug("request %s %s" % (path, ext))
+        log.debug("request %s %s" % (path, ext))
         log.debug(cherrypy.request.headers)
         accept = {}
         if content_type is None:
@@ -469,22 +531,22 @@ class MDServer():
                     if r is not None:
                         cache_ttl = state.get('cache', 0)
                         log.debug("caching for %d seconds" % cache_ttl)
-                        caching.expires(secs=cache_ttl)
                         for k, v in state.get('headers', {}).iteritems():
                             cherrypy.response.headers[k] = v
+                        caching.expires(secs=cache_ttl)
                         return r
         raise NotFound()
 
 
 def main():
     """
-    The main entrypoint for the pyFF mdx server.
+    The main entrypoint for the pyffd command.
     """
     try:
         opts, args = getopt.getopt(sys.argv[1:],
                                    'hP:p:H:CfaA:l:',
                                    ['help', 'loglevel=', 'log=', 'access-log=', 'error-log=', 'port=', 'host=',
-                                    'no-caching', 'autoreload', 'frequency=', 'alias=', 'dir='])
+                                    'no-caching', 'autoreload', 'frequency=', 'alias=', 'dir=', 'version'])
     except getopt.error, msg:
         print msg
         print __doc__
@@ -529,7 +591,7 @@ def main():
             elif o in ('--no-caching', '-C'):
                 caching = False
             elif o in ('--caching-delay', 'D'):
-                delay = int(caching)
+                delay = int(o)
             elif o in ('--foreground', '-f'):
                 daemonize = False
             elif o in ('--autoreload', '-a'):
@@ -542,8 +604,11 @@ def main():
                     aliases[a] = uri
             elif o in '--dir':
                 base_dir = a
+            elif o in '--version':
+                print "pyffd version %s (cherrypy version %s)" % (pyff_version, cherrypy.__version__)
+                sys.exit(0)
             else:
-                raise ValueError("Unknown option %s" % o)
+                raise ValueError("Unknown option '%s'" % o)
 
     except Exception, ex:
         print ex
@@ -580,17 +645,16 @@ def main():
 
     server = MDServer(pipes=args, autoreload=autoreload, frequency=frequency, aliases=aliases, cache_enabled=caching)
     pfx = ["/entities", "/metadata"] + ["/" + x for x in server.aliases.keys()]
-
     cfg = {
         'global': {
             'server.socket_port': port,
             'server.socket_host': host,
             'tools.caching.on': caching,
-            'tools.caching.debug': True,
+            'tools.caching.debug': caching,
             'tools.trailing_slash.on': True,
-            'tools.caching.maxobj_size': 1000000000000, # effectively infinite
+            'tools.caching.maxobj_size': 1000000000000,  # effectively infinite
             'tools.caching.maxsize': 1000000000000,
-            'tools.caching.antistampede_timeout': None,
+            'tools.caching.antistampede_timeout': 30,
             'tools.caching.delay': 3600,  # this is how long we keep static stuff
             'tools.cpstats.on': True,
             'error_page.404': lambda **kwargs: error_page(404, **kwargs),
